#include <windows.h>
#include <shlobj.h>
#include "fish.h"
#pragma hdrstop

extern HINSTANCE hinst;

int Here(ulong) { return 0; }

#if 1
static const char a_askdir[] = "Entrez le nom d'un nouveau rpertoire\n"
                               "Il sera cr dans %s";
static const char a_cantcreatedir[] = "Impossible de crer le rpertoire %s";
static const char a_nomem[] = "Pas assez de mmoire pour cette opration (%s)";
static const char a_newdir[] = "Nouveau rpertoire";
#else
static const char a_askdir[] = "Please enter a new directory name\n"
                               "It will be created in %s";
static const char a_cantcreatedir[] = "Can not create directory %s";
static const char a_nomem[] = "Not enough memory for operation (%s)";
static const char a_newdir[] = "New directory";
#endif

//------------------------------------------------------------------------
void vwarning(bool iserror, const char *format, va_list va)
{
  char buf[2*MAXSTR];
  wvsprintf(buf, format, va);
  MessageBox(NULL, buf, appname, (iserror ? MB_ICONERROR : MB_ICONWARNING)|MB_OK);
}

//------------------------------------------------------------------------
void warning(const char *format, ...)
{
  va_list va;
  va_start(va, format);
  vwarning(false, format, va);
  va_end(va);
}

//------------------------------------------------------------------------
void error(const char *format, ...)
{
  va_list va;
  va_start(va, format);
  vwarning(true, format, va);
  va_end(va);
  ExitProcess(-1);
}

//------------------------------------------------------------------------
void nomem(const char *where)
{
  error(a_nomem, where);
}

//------------------------------------------------------------------------
struct askstr_t
{
  char *question;
  char *answer;
};

#define ASKSTR_LABEL    101
#define ASKSTR_EDIT     102

BOOL CALLBACK AskStrProc(HWND hwnd, UINT message, WPARAM wp, LPARAM lp)
{
  askstr_t *as;
  switch ( message )
  {
    case WM_INITDIALOG:
      {
        as = (askstr_t *)lp;
        SendDlgItemMessage(hwnd, ASKSTR_EDIT, EM_SETLIMITTEXT, MAX_PATH-1, 0);
        SetDlgItemText(hwnd, ASKSTR_EDIT, as->answer);
        SetDlgItemText(hwnd, ASKSTR_LABEL, as->question);
        SetWindowLong(hwnd, GWL_USERDATA, lp);
      }  
      return TRUE;

    case WM_COMMAND:
      as = (askstr_t *)GetWindowLong(hwnd, GWL_USERDATA);
      switch ( LOWORD(wp) )
      {
        case IDOK:
          GetDlgItemText(hwnd, ASKSTR_EDIT, as->answer, MAX_PATH);
          EndDialog(hwnd, 1);
          return TRUE;
        case IDCANCEL:
          EndDialog(hwnd, 0);
          return TRUE;
      }
      break;

  }
  return FALSE;
}


bool vaskstr(HWND wnd, char *str, const char *format, va_list va)
{
  char buf[2*MAXSTR];
  wvsprintf(buf, format, va);
  askstr_t as;
  as.question = buf;
  as.answer   = str;
  int ok = DialogBoxParam(hinst, MAKEINTRESOURCE(3), wnd, (DLGPROC)AskStrProc, LPARAM(&as));
  return ok == 1;
}

//------------------------------------------------------------------------
bool askstr(HWND wnd, char *str, const char *format, ...)
{
  va_list va;
  va_start(va, format);
  bool ok = vaskstr(wnd, str, format, va);
  va_end(va);
  return ok;
}

//------------------------------------------------------------------------
char *b2a(const uchar *b, char *a) // binary to ascii
{
  char *o = a;
  for ( int i=0; i < KEYSIZE; i++ )
  {
    static char digits[] = "0123456789ABCDEF";
    uchar c = b[i];
    *o++ = digits[c>>4];
    *o++ = digits[c&15];
  }
  *o = '\0';
  return a;
}

//------------------------------------------------------------------------
/*
**  hashpass - Hash pass phrase down to 128 bits (16 bytes).
**  keylen must be less than 1024.
**  Use the MD5 algorithm.
*/
void _hashpass(const char *keystring, int keylen, uchar *hash)
{
  struct MD5Context mdContext;

  /* Calculate the hash */
  MD5Init(&mdContext);
  MD5Update(&mdContext, (const unsigned char *) keystring, keylen);
  MD5Final(hash, &mdContext);
}

void hashpass(const char *keystring, int keylen, uchar *hash)
{
  _hashpass(keystring, keylen, hash);
  for ( int i=0; i < 1000; i++ ) _hashpass((char*)hash, KEYSIZE, hash);
}

//------------------------------------------------------------------------
/* Table of CRC-32's of all single byte values (made by makecrc.c) */
static const ulong crc_32_tab[] = {
  0x00000000L, 0x77073096L, 0xee0e612cL, 0x990951baL, 0x076dc419L,
  0x706af48fL, 0xe963a535L, 0x9e6495a3L, 0x0edb8832L, 0x79dcb8a4L,
  0xe0d5e91eL, 0x97d2d988L, 0x09b64c2bL, 0x7eb17cbdL, 0xe7b82d07L,
  0x90bf1d91L, 0x1db71064L, 0x6ab020f2L, 0xf3b97148L, 0x84be41deL,
  0x1adad47dL, 0x6ddde4ebL, 0xf4d4b551L, 0x83d385c7L, 0x136c9856L,
  0x646ba8c0L, 0xfd62f97aL, 0x8a65c9ecL, 0x14015c4fL, 0x63066cd9L,
  0xfa0f3d63L, 0x8d080df5L, 0x3b6e20c8L, 0x4c69105eL, 0xd56041e4L,
  0xa2677172L, 0x3c03e4d1L, 0x4b04d447L, 0xd20d85fdL, 0xa50ab56bL,
  0x35b5a8faL, 0x42b2986cL, 0xdbbbc9d6L, 0xacbcf940L, 0x32d86ce3L,
  0x45df5c75L, 0xdcd60dcfL, 0xabd13d59L, 0x26d930acL, 0x51de003aL,
  0xc8d75180L, 0xbfd06116L, 0x21b4f4b5L, 0x56b3c423L, 0xcfba9599L,
  0xb8bda50fL, 0x2802b89eL, 0x5f058808L, 0xc60cd9b2L, 0xb10be924L,
  0x2f6f7c87L, 0x58684c11L, 0xc1611dabL, 0xb6662d3dL, 0x76dc4190L,
  0x01db7106L, 0x98d220bcL, 0xefd5102aL, 0x71b18589L, 0x06b6b51fL,
  0x9fbfe4a5L, 0xe8b8d433L, 0x7807c9a2L, 0x0f00f934L, 0x9609a88eL,
  0xe10e9818L, 0x7f6a0dbbL, 0x086d3d2dL, 0x91646c97L, 0xe6635c01L,
  0x6b6b51f4L, 0x1c6c6162L, 0x856530d8L, 0xf262004eL, 0x6c0695edL,
  0x1b01a57bL, 0x8208f4c1L, 0xf50fc457L, 0x65b0d9c6L, 0x12b7e950L,
  0x8bbeb8eaL, 0xfcb9887cL, 0x62dd1ddfL, 0x15da2d49L, 0x8cd37cf3L,
  0xfbd44c65L, 0x4db26158L, 0x3ab551ceL, 0xa3bc0074L, 0xd4bb30e2L,
  0x4adfa541L, 0x3dd895d7L, 0xa4d1c46dL, 0xd3d6f4fbL, 0x4369e96aL,
  0x346ed9fcL, 0xad678846L, 0xda60b8d0L, 0x44042d73L, 0x33031de5L,
  0xaa0a4c5fL, 0xdd0d7cc9L, 0x5005713cL, 0x270241aaL, 0xbe0b1010L,
  0xc90c2086L, 0x5768b525L, 0x206f85b3L, 0xb966d409L, 0xce61e49fL,
  0x5edef90eL, 0x29d9c998L, 0xb0d09822L, 0xc7d7a8b4L, 0x59b33d17L,
  0x2eb40d81L, 0xb7bd5c3bL, 0xc0ba6cadL, 0xedb88320L, 0x9abfb3b6L,
  0x03b6e20cL, 0x74b1d29aL, 0xead54739L, 0x9dd277afL, 0x04db2615L,
  0x73dc1683L, 0xe3630b12L, 0x94643b84L, 0x0d6d6a3eL, 0x7a6a5aa8L,
  0xe40ecf0bL, 0x9309ff9dL, 0x0a00ae27L, 0x7d079eb1L, 0xf00f9344L,
  0x8708a3d2L, 0x1e01f268L, 0x6906c2feL, 0xf762575dL, 0x806567cbL,
  0x196c3671L, 0x6e6b06e7L, 0xfed41b76L, 0x89d32be0L, 0x10da7a5aL,
  0x67dd4accL, 0xf9b9df6fL, 0x8ebeeff9L, 0x17b7be43L, 0x60b08ed5L,
  0xd6d6a3e8L, 0xa1d1937eL, 0x38d8c2c4L, 0x4fdff252L, 0xd1bb67f1L,
  0xa6bc5767L, 0x3fb506ddL, 0x48b2364bL, 0xd80d2bdaL, 0xaf0a1b4cL,
  0x36034af6L, 0x41047a60L, 0xdf60efc3L, 0xa867df55L, 0x316e8eefL,
  0x4669be79L, 0xcb61b38cL, 0xbc66831aL, 0x256fd2a0L, 0x5268e236L,
  0xcc0c7795L, 0xbb0b4703L, 0x220216b9L, 0x5505262fL, 0xc5ba3bbeL,
  0xb2bd0b28L, 0x2bb45a92L, 0x5cb36a04L, 0xc2d7ffa7L, 0xb5d0cf31L,
  0x2cd99e8bL, 0x5bdeae1dL, 0x9b64c2b0L, 0xec63f226L, 0x756aa39cL,
  0x026d930aL, 0x9c0906a9L, 0xeb0e363fL, 0x72076785L, 0x05005713L,
  0x95bf4a82L, 0xe2b87a14L, 0x7bb12baeL, 0x0cb61b38L, 0x92d28e9bL,
  0xe5d5be0dL, 0x7cdcefb7L, 0x0bdbdf21L, 0x86d3d2d4L, 0xf1d4e242L,
  0x68ddb3f8L, 0x1fda836eL, 0x81be16cdL, 0xf6b9265bL, 0x6fb077e1L,
  0x18b74777L, 0x88085ae6L, 0xff0f6a70L, 0x66063bcaL, 0x11010b5cL,
  0x8f659effL, 0xf862ae69L, 0x616bffd3L, 0x166ccf45L, 0xa00ae278L,
  0xd70dd2eeL, 0x4e048354L, 0x3903b3c2L, 0xa7672661L, 0xd06016f7L,
  0x4969474dL, 0x3e6e77dbL, 0xaed16a4aL, 0xd9d65adcL, 0x40df0b66L,
  0x37d83bf0L, 0xa9bcae53L, 0xdebb9ec5L, 0x47b2cf7fL, 0x30b5ffe9L,
  0xbdbdf21cL, 0xcabac28aL, 0x53b39330L, 0x24b4a3a6L, 0xbad03605L,
  0xcdd70693L, 0x54de5729L, 0x23d967bfL, 0xb3667a2eL, 0xc4614ab8L,
  0x5d681b02L, 0x2a6f2b94L, 0xb40bbe37L, 0xc30c8ea1L, 0x5a05df1bL,
  0x2d02ef8dL
};


ulong crc32(void *buf,int n)
//char *s;                /* pointer to bytes to pump through */
//extent n;               /* number of bytes in s[] */
/* Run a set of bytes through the crc shift register.  If s is a NULL
   pointer, then initialize the crc shift register contents instead.
   Return the current crc in either case. */
{
  register ulong c;       /* temporary variable */

  static ulong crc = 0xffffffffL; /* shift register contents */

  char *s = (char *)buf;
  if (s == NULL)
    c = 0xffffffffL;
  else
  {
    c = crc;
    while (n--)
//      c = CRC32(c, *s++);
      c = crc_32_tab[((int)c ^ (*s++)) & 0xff] ^ (c >> 8);
  }
  crc = c;
  return c ^ 0xffffffffL;       /* (instead of ~c for 64-bit machines) */
}

//---------------------------------------------------------------------------
// Center the given window on the Foreground window
void CenterWindow(HWND hwnd)
{
  RECT Rect;
  RECT FWRect;
  HWND ForegroundWindow;
  
  ForegroundWindow = GetForegroundWindow();
  GetWindowRect(hwnd, &Rect);
  GetWindowRect(ForegroundWindow, &FWRect);
  SetWindowPos(hwnd,0,
		(FWRect.left+(FWRect.right-FWRect.left)/2)-(Rect.right-Rect.left)/2,
		(FWRect.top+(FWRect.bottom-FWRect.top)/2)-(Rect.bottom-Rect.top)/2,
		0,0,SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOZORDER);
}

//--------------------------------------------------------------------------------------------------------//
wchar_t *StringToOleStr(const char *string)
{
  wchar_t buf[1024];
  const int maxchars = sizeof(buf)/sizeof(wchar_t);

  int len = strlen(string);
  if ( len < maxchars )
  {
    return SysAllocStringLen(buf,
        MultiByteToWideChar(0, 0, string, len, buf, maxchars));
  }
  else
  {
    int rlen = MultiByteToWideChar(0, 0, string, len, NULL, 0);
    wchar_t *res = SysAllocStringLen(NULL, rlen);
    MultiByteToWideChar(0, 0, string, len, res, rlen);
    return res;
  }
}

//--------------------------------------------------------------------------------------------------------//
bool file_exists(const char *file)
{
  return GetFileAttributes(file) != DWORD(-1);
}

//--------------------------------------------------------------------------------------------------------//
bool is_directory(const char *file)
{
  DWORD attr = GetFileAttributes(file);
  return attr != DWORD(-1) && (attr & FILE_ATTRIBUTE_DIRECTORY) != 0;
}

//--------------------------------------------------------------------------------------------------------//
// This procedure compliments SHGetPathFromIDList in the Win32 API, and allows you to obtain a PIDL
// from a path string.
void GetIDListFromPath(const char *Path, LPITEMIDLIST *PItemIDList)
{
  IShellFolder *ShellFolderInterface;
  ULONG CharsParsed;
  ULONG Attributes;

  PItemIDList = NULL;
  // Get a pointer to the/an IShellFolder interface.  The easiest way to do this is
  // to use SHGetDesktopFolder
  if ( SHGetDesktopFolder(&ShellFolderInterface) == NOERROR )
  {
    // Generate a PIDL from the given path name
    // Don't forget, you'll need to free the ItemIDLIst using
    //	CoTaskMemFree(ItemIDList)
    if ( file_exists(Path) )
    {
      wchar_t *olepath = StringToOleStr(Path);
      ShellFolderInterface->ParseDisplayName(0, NULL,
                                       olepath,
                                       &CharsParsed,
                                       PItemIDList,
                                       &Attributes);
      SysFreeString(olepath);
    }
    // Release the interface.  Always necessary with COM interfaces.
    ShellFolderInterface->Release();
  }
}

//---------------------------------------------------------------------------
struct browser_lpdata_t
{
  char *inipath;
  char curpath[MAX_PATH];
};

#define CUSTOM_BUTTON_ID 1234

// A custom window procedure to process button clicks
static LRESULT CALLBACK CustomWndProc(

    HWND hwnd,     	// handle of window
    UINT message,	// message identifier
    WPARAM wParam,	// first message parameter
    LPARAM lParam 	// second message parameter
   )
{
  browser_lpdata_t *lp = (browser_lpdata_t *)GetWindowLong(hwnd, GWL_USERDATA);
  // Intercept our custom button's click events
  if ( message == WM_COMMAND
    && LOWORD(wParam) == CUSTOM_BUTTON_ID
    && lp->curpath[0] != '\0' )
  {
    char newdir[MAX_PATH];
    char subdir[MAX_PATH];
    subdir[0] = '\0';
    strcpy(newdir, lp->curpath);
    if ( askstr(hwnd, subdir, a_askdir, newdir) )
    {
      char *end;
      if ( subdir[1] == ':' || subdir[0] == '\\' )
      {
        // full path is specified, don't change it
        end = newdir;
      }
      else
      {
        end = strchr(newdir, 0);
        if ( end[-1] != '\\' ) *end++ = '\\';
      }  
      strcpy(end, subdir);
      if ( !CreateDirectory(newdir, NULL) )
      {
        warning(a_cantcreatedir, newdir);
      }  
      else
      {
        strcpy(lp->curpath, newdir);
        SendMessage(hwnd, WM_COMMAND, IDOK, 0);
      }  
    }  
    return 1;
  }      
  return DefDlgProc(hwnd, message, wParam, lParam);
}

//---------------------------------------------------------------------------
static HWND AddButton(HWND hwnd, const char *caption)
{
  // Create the custom button
  int style = WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON;
  HWND button = CreateWindow("BUTTON", caption, style, 12, 270, 50, 23, hwnd,
                    HMENU(CUSTOM_BUTTON_ID), hinst, NULL);

  // Ensure the button's font is the default system font
  HGDIOBJ sysfont = GetStockObject(DEFAULT_GUI_FONT);
  SendMessage(button, WM_SETFONT, LPARAM(sysfont), MAKELPARAM(1,0));

  // calculate the button width
  HDC hdc = GetDC(button);
  SIZE s;
  GetTextExtentPoint32(hdc, caption, strlen(caption), &s);  
  ReleaseDC(button, hdc);
  int width = s.cx + 6;
  
  // change the button width
  SetWindowPos(button, 0, 0, 0, width, 23, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOZORDER);

  // Replace the dialog's window procedure with our custom window procedure
  SetWindowLong(hwnd, GWL_WNDPROC, LONG(CustomWndProc));
  return button;
}

//---------------------------------------------------------------------------
// Callback procedure; Windows calls this procedure upon certain events in the
// browse dialog.
static int CALLBACK BrowserCallback(
    HWND hwnd,     	// handle of window
    UINT uMsg,  	// message identifier
    LPARAM lParam,	// first message parameter
    LPARAM lpData 	// second message parameter
   )
{
  browser_lpdata_t *lp;
  // Process the standard dialog messages
  switch ( uMsg )
  {
    case BFFM_INITIALIZED:
      lp = (browser_lpdata_t *)lpData;
      SetWindowLong(hwnd, GWL_USERDATA, lpData);
      CenterWindow(hwnd);

      // Set the initial path
      {
        char *inipath = lp->inipath;
        if ( inipath != NULL && is_directory(inipath) )
          SendMessage(hwnd, BFFM_SETSELECTION, true, LPARAM(inipath));
      }    
      AddButton(hwnd, a_newdir);
      break;
    case BFFM_SELCHANGED:
      lp = (browser_lpdata_t *)lpData;
      SHGetPathFromIDList(LPITEMIDLIST(lParam), lp->curpath);
      break;
  }
  return 0;
}

//---------------------------------------------------------------------------
// hwnd - handle of the application's main window
//
// Uses the global variable g_pMalloc, which is assumed to point
//     to the shell's IMalloc interface.
bool BrowseForFolder(HWND hwnd, const char *title, char *buf)
{
    BROWSEINFO bi;
    LPSTR lpBuffer;
    LPITEMIDLIST pidlBrowse;    // PIDL selected by user
    bool ok = false;

    static LPMALLOC g_pMalloc = NULL;
    if ( g_pMalloc == NULL ) SHGetMalloc(&g_pMalloc);
    if ( g_pMalloc == NULL ) return false;

    // Allocate a buffer to receive browse information.

    if ((lpBuffer = (LPSTR) g_pMalloc->Alloc(MAX_PATH)) == NULL)
        return false;

    browser_lpdata_t lp;
    lp.inipath = buf[0] ? buf : NULL;
    lp.curpath[0] = '\0';
    // Fill in the BROWSEINFO structure.
    bi.hwndOwner = hwnd;
    bi.pidlRoot = NULL;
    bi.pszDisplayName = lpBuffer;

    bi.lpszTitle = title;
    bi.ulFlags = BIF_RETURNONLYFSDIRS;  // BIF_USENEWUI
    bi.lpfn = BrowserCallback;
    bi.lParam = LONG(&lp);

    // Browse for a folder and return its PIDL.
    pidlBrowse = SHBrowseForFolder(&bi);
    if ( pidlBrowse != NULL )
    {
      strcpy(buf, lp.curpath);
      ok = true;
//      if (SHGetPathFromIDList(pidlBrowse, lpBuffer))
//      {
//        strcpy(buf, lpBuffer);
//        ok = true;
//      }
      g_pMalloc->Free(pidlBrowse);
    }
    g_pMalloc->Free(lpBuffer);
    return ok;
}

//---------------------------------------------------------------------------
#if 0 // we need newer SDK to compile this
#define PACKVERSION(major,minor) MAKELONG(minor,major)

DWORD GetDllVersion(LPCTSTR lpszDllName)
{

    HINSTANCE hinstDll;
    DWORD dwVersion = 0;

    hinstDll = LoadLibrary(lpszDllName);

    if(hinstDll)
    {
        DLLGETVERSIONPROC pDllGetVersion;

        pDllGetVersion = (DLLGETVERSIONPROC) GetProcAddress(hinstDll, "DllGetVersion");

/*Because some DLLs may not implement this function, you
 *must test for it explicitly. Depending on the particular
 *DLL, the lack of a DllGetVersion function may
 *be a useful indicator of the version.
*/
        if(pDllGetVersion)
        {
            DLLVERSIONINFO dvi;
            HRESULT hr;

            ZeroMemory(&dvi, sizeof(dvi));
            dvi.cbSize = sizeof(dvi);

            hr = (*pDllGetVersion)(&dvi);

            if(SUCCEEDED(hr))
            {
                dwVersion = PACKVERSION(dvi.dwMajorVersion, dvi.dwMinorVersion);
            }
        }

        FreeLibrary(hinstDll);
    }
    return dwVersion;
}


/*
The following code fragment illustrates how you can use GetDllVersion to test if Comctl32.dll is version 4.71 or later.

if(GetDllVersion(TEXT("comctl32.dll")) >= PACKVERSION(4,71))
{
    //Proceed
}
else
{
    //Use an alternate approach for older DLL versions
}
*/
#endif // 0