#include <windows.h>

static const char *msg;
//----------------------------------------------------------------
LRESULT CALLBACK WndProc(

    HWND hwnd,	// handle of window
    UINT message,	// message identifier
    WPARAM wParam,	// first message parameter
    LPARAM lParam 	// second message parameter
   )
{
   HDC hdc;
   PAINTSTRUCT ps;
   RECT rect;
   HBRUSH hb;

   switch ( message )
   {
     case WM_CREATE:
       hdc = GetDC(hwnd);
       GetClientRect(hwnd, &rect);
       DrawText(hdc, msg, -1, &rect,
                 DT_CALCRECT | DT_LEFT | DT_NOCLIP | DT_NOPREFIX | DT_WORDBREAK );
       SetWindowPos(hwnd, HWND_TOP,
                0, 0,
                rect.right+4, rect.bottom+4,
                SWP_NOMOVE|SWP_NOOWNERZORDER);
       ReleaseDC(hwnd, hdc);
       return 0;

     case WM_PAINT:
       {
         hdc = BeginPaint(hwnd, &ps);
         GetClientRect(hwnd, &rect);
         COLORREF bg = GetSysColor(COLOR_INFOBK);
         COLORREF fg = GetSysColor(COLOR_INFOTEXT);

         hb = CreateSolidBrush(bg);
         FillRect(hdc, &rect, hb);
         DeleteObject(hb);
         
         SetBkColor(hdc, bg);
         SetTextColor(hdc, fg);
         rect.left++;
         rect.top++;
         DrawText(hdc, msg, -1, &rect,
                 DT_LEFT | DT_NOCLIP | DT_NOPREFIX | DT_WORDBREAK );
         rect.left--;
         rect.top--;
                 
         hb = CreateSolidBrush(fg);
         FrameRect(hdc, &rect, hb);
         DeleteObject(hb);
         
         EndPaint(hwnd, &ps);
       }  
       return 0;

     case WM_DESTROY:
       PostQuitMessage(0);
       return 0;
  }
  return DefWindowProc(hwnd, message, wParam, lParam);
}

//----------------------------------------------------------------
HWND create_hint(HINSTANCE h, HWND hwnd_parent, int x, int y, const char *message)
{
  WNDCLASS wc;
  static const char wndclass[] = "HintWindowClass";

  msg = message;
  
  // Register the window class for the hint window.

  wc.style = CS_HREDRAW|CS_VREDRAW;
  wc.lpfnWndProc = (WNDPROC) WndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = h;

  wc.hIcon   = NULL;
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = NULL;
  wc.lpszMenuName  = NULL;
  wc.lpszClassName = wndclass;

  if (!RegisterClass(&wc)) return 0;

  // Create the hint window.

  HWND hwnd = CreateWindow(wndclass, "decryptor hint",
      WS_POPUP,
      x,
      y,
      x+10,
      y+1,
//      CW_USEDEFAULT,
//      CW_USEDEFAULT,
      hwnd_parent,
      (HMENU) NULL, h, (LPVOID) NULL);

  // If the hint window cannot be created, return
  // the application.

  if (!hwnd) return 0;

  // Show the window and paint its contents.

  ShowWindow(hwnd, SW_SHOWNORMAL);
  UpdateWindow(hwnd);

  return hwnd;
}
