//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <process.h>
#include <shellapi.h>
#include <MAPI.hpp>
#include <dir.h>
#include <sys/stat.h>
#include "fish.h"
#include "Unit1.h"
#include "Unit2.h"
#include "Unit3.h"
#include "option.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "DropSource"
#pragma resource "*.dfm"

// The program will work till this date:
#define DOOMDAY 0x3a273050 // user: 00:00:00 01-12-2000
//#define DOOMDAY 0x3844ab50 // user: 00:00:00 01-12-1999

// ID of the trayicon. The value
// is arbitrary. If the program displays more than one tray
// icon at a time, choose a unique ID for each icon.
// The char * constant is the hint text for the tray icon.
// The last constant is a user defined message that the tray
// will send to our window handle for tray mouse events.
const int IDC_TRAY1      = 1005;

// Program messages to make internationalization easier
const char *HINT_MESSAGE   = "Open aCrypt";    // tray hint
const char url[]           = "http://www.acrypt.com";
const char a_expired[]     = "This program has expired. Please visit %s to download a new version";
const char a_compressing[] = "Compressing...";
const char a_encrypting[]  = "Encrypting...";
const char a_writing[]     = "Writing...";
const char a_encrypted[]   = "The file %s is already encrypted";
const char a_overwrite[]   = "The output file %s already exists. Do you want to overwrite it?";
const char a_cantopen[]    = "Can not open file %s (code %d)";
const char a_nocreate[]    = "Can not create the encrypted file %s (code %d)";
const char a_read[]        = "Read error on %s (code %d)";
const char a_write[]       = "Write error on %s (code %d)";
const char a_nostub[]      = "Essential files are missing - please reinstall aCrypt";
      char a_badstub[]     = "Essential files corrupted (CRC %08lX)";
const char a_regerror[]    = "Could not create registry key, the configuration has not been saved";
const char a_many[]        = "Encrypt several files";  // title of passphrase dialog

// Name of the file with the executable stub - not used in the final version
const char stub[]          = "stub";

// The output files are stored here
char encrypted_folder[MAX_PATH] = "c:\\encrypted\\";

// show icon on systray by default
DWORD ctray = 1;

// List of disable task windows - used to re-enable them at the end of encryption
static HWND WindowList;

TForm1 *Form1;

//---------------------------------------------------------------------------
// Complain about a problem with the registry.
void registry_error(void)
{
    warning(a_regerror);
}

//---------------------------------------------------------------------------
// Create the main program form
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
  // fill the "appname" with the appliation name
  // this variable will be used by some functions common for the decryptor
  // and encryptor.
  strcpy(appname,Application->Title.c_str());
  // Double buffer the file list to redraw faster and without flickers
  ListView1->DoubleBuffered = true;
}


//---------------------------------------------------------------------------
// Destroy the main form
__fastcall TForm1::~TForm1()
{
    // Remove the icon from the tray, and delete
    // the TIcon pointer that we initially created.
    RemoveIcon();
    delete TrayIcon;
}

//-------------------------------------------------------
// This function is called when WM_TRAYNOTIFY message is received
void __fastcall TForm1::WMTrayNotify(TMessage &Msg)
{
    // The LPARAM of the message identifies the type of mouse message.
    // When they right click, show the popup menu. When they double
    // click with the left mouse, show the form.
    switch(Msg.LParam)
    {
        case WM_RBUTTONUP:
            // Find the mouse cursor and popup the popupmenu at that
            // location. The SetForegroundWindow and PostMessage calls
            // fix an OS bug that prevents the menu from closing when
            // the user clicks outside the menu. KB article Q135788
            POINT WinPoint;
            GetCursorPos(&WinPoint);
            SetForegroundWindow(Handle);
            TrayMenu->Popup(WinPoint.x,WinPoint.y);
            PostMessage(Handle, WM_NULL, 0,0);
            break;
        case WM_LBUTTONDBLCLK:
            // Restore the main form
            Visible = true;
            ShowWindow(Application->Handle, SW_SHOW);
            PostMessage(Application->Handle, WM_SYSCOMMAND, SC_RESTORE, 0);
            SetForegroundWindow(Handle);
            break;
    }
}

//-------------------------------------------------------
// Add an icon to the system tray
void __fastcall TForm1::AddIcon()
{
    // Use the Shell_NotifyIcon API function to
    // add the icon to the tray.
    NOTIFYICONDATA IconData;
    IconData.cbSize = sizeof(NOTIFYICONDATA);
    IconData.uID    = IDC_TRAY1;
    IconData.hWnd   = Handle;
    IconData.uFlags = NIF_MESSAGE|NIF_ICON|NIF_TIP;
    IconData.uCallbackMessage = WM_TRAYNOTIFY;
    lstrcpy(IconData.szTip, HINT_MESSAGE);
    IconData.hIcon  = TrayIcon->Handle;

    Shell_NotifyIcon(NIM_ADD,&IconData);
}

//-------------------------------------------------------
// Remove the icon from the system tray
void __fastcall TForm1::RemoveIcon()
{
    NOTIFYICONDATA IconData;
    IconData.cbSize = sizeof(NOTIFYICONDATA);
    IconData.uID    = IDC_TRAY1;
    IconData.hWnd   = Handle;
    IconData.hIcon  = TrayIcon->Handle;

    Shell_NotifyIcon(NIM_DELETE,&IconData);
}

//-------------------------------------------------------
// The user tries to close the main form
// If allowed, minimize to the system tray
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
    // This application closes to the system tray. When they push
    // close, hide the app, but don't let the program close.

  if ( !Application->Terminated && ctray && GetKeyState(VK_SHIFT) >= 0 )
  {
    ShowWindow(Application->Handle, SW_HIDE);
    Visible = false;
    Action = caNone;
  }
}

//---------------------------------------------------------------------------
// Initialize the program when the main form is created
void __fastcall TForm1::FormCreate(TObject *Sender)
{
  // The main form can not occupy the whole screen
  Constraints->MaxHeight = Screen->Height / 2;
  Constraints->MaxWidth  = Screen->Width / 2;
  // Read the window position from the registry
  read_cfg_position();
  // Read the configuration from the registry.
  // The passphrase and "save" are discarded here.
  bool save;
  char phrase[MAXSTR];
  read_config(&save, phrase);
  populate_images();
  populate_list(encrypted_folder);
  DragAcceptFiles(ListView1->Handle, TRUE);
  Application->OnMessage = AppOnMessage;
  Caption = Application->Title;
#ifdef DOOMDAY
  if ( time(NULL) >= DOOMDAY )
    error(a_expired, url);
#endif
  TrayIcon = new Graphics::TIcon;
  TrayIcon->Handle=LoadImage(HInstance,
                             "MAINICON",
                             IMAGE_ICON,
                             0,0,
                             0);
  // Add the icon to the taskbar
  AddIcon();
}

//---------------------------------------------------------------------------
// Returns the file name without directory part.
const char *basename(const char *fname)
{
  const char *p = strrchr(fname, '\\');
  if ( p == NULL )
  {
    p = strchr(fname, ':');
    if ( p == NULL ) return fname;
  }
  return p+1;
}

//---------------------------------------------------------------------------
// Ask for a file and encrypt it
void __fastcall TForm1::BrowseExecute(TObject *Sender)
{
  if ( !OpenDialog1->Execute() ) return;
  AnsiString files = "";
  for ( int i=0; i < OpenDialog1->Files->Count; i++ )
  {
    if ( i != 0 ) files += ";";
    files += OpenDialog1->Files->Strings[i];
  }
  encrypt(files.c_str());
}

//---------------------------------------------------------------------------
// The help button is pressed
void __fastcall TForm1::HelpExecute(TObject *Sender)
{
  Application->HelpContext(HelpContext);
}

//---------------------------------------------------------------------------
// The URL is clicked
void __fastcall TForm1::Label1Click(TObject *Sender)
{
  wwwExecute(Sender);
}

//---------------------------------------------------------------------------
// Refresh the file list
void __fastcall TForm1::populate_list(const char *folder)
{
  // The directory name is assumed to end with a backslash.

  WIN32_FIND_DATA filedata;  // Structure for file data
  HANDLE filehandle;         // Handle for searching

  ListView1->Items->BeginUpdate();
  ListView1->Items->Clear();

  AnsiString allfiles = AnsiString(folder) + "*.*";
  filehandle = FindFirstFile (allfiles.c_str(), &filedata);
  if (filehandle != INVALID_HANDLE_VALUE)
  {
    do
    {
      if ((filedata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) != 0 )
        continue;
      // Display the file name.
      TListItem *li = ListView1->Items->Add () ;
      li->Caption = filedata.cFileName ;

      // Display the file size.
      if (filedata.nFileSizeHigh == 0)
      {
        li->SubItems->Add(String(filedata.nFileSizeLow));
      }
      else
      {
        // If nFileSizeHigh is non-zero then this is a **HUGE** file.
        double filesize = (MAXDWORD * filedata.nFileSizeHigh) + filedata.nFileSizeLow;
        li->SubItems->Add(String(filesize / 1024.0) + " K");
      }

      // get icon
      SHFILEINFO info;
      AnsiString full = AnsiString(folder) + filedata.cFileName;
      DWORD result = SHGetFileInfo(full.c_str(),
                                   0,
                                   &info,
                                   sizeof(info),
                                   SHGFI_ICON |
                                   SHGFI_SHELLICONSIZE |
                                   SHGFI_SYSICONINDEX);

      // Check for failure. If return value was OK, the icon index
      // is return in the iIcon member of the SHFILEINFO structure.
      if ( result != 0 ) li->ImageIndex = info.iIcon;

      Application->ProcessMessages();

    } while (FindNextFile(filehandle, &filedata));
    FindClose(filehandle);
  }
  ListView1->Items->EndUpdate();
  Visible = true;
}


//---------------------------------------------------------------------------
// Initialize the image lists: normal icons and small icons
void __fastcall TForm1::populate_images(void)
{
   SHFILEINFO info;
   DWORD ImageHandle = SHGetFileInfo("",
                                     0,
                                     &info,
                                     sizeof(info),
                                     SHGFI_ICON |
                                     SHGFI_SHELLICONSIZE |
                                     SHGFI_SYSICONINDEX);

   if (ImageHandle != 0)
   {
     ImageList1->Handle = ImageHandle;
     ImageList1->ShareImages = true;
   }
         ImageHandle = SHGetFileInfo("",
                                     0,
                                     &info,
                                     sizeof(info),
                                     SHGFI_SMALLICON |
                                     SHGFI_SHELLICONSIZE |
                                     SHGFI_SYSICONINDEX);

   if (ImageHandle != 0)
   {
     ImageList2->Handle = ImageHandle;
     ImageList2->ShareImages = true;
   }
}

//---------------------------------------------------------------------------
// Use large icon format in the file list
void __fastcall TForm1::LargeIconExecute(TObject *Sender)
{
  ListView1->ViewStyle = vsIcon;
  LargeIcons1->Checked = true;
  LargeIcons2->Checked = true;
}

// Use small icon format in the file list
void __fastcall TForm1::SmallIconExecute(TObject *Sender)
{
  ListView1->ViewStyle = vsSmallIcon;
  SmallIcons1->Checked = true;
  SmallIcons2->Checked = true;
}

// Use detailed view in the file list
void __fastcall TForm1::DetailsExecute(TObject *Sender)
{
  ListView1->ViewStyle = vsReport;
  Details1->Checked = true;
  Details2->Checked = true;
}

// Use list view in the file list
void __fastcall TForm1::ListExecute(TObject *Sender)
{
  ListView1->ViewStyle = vsList;
  List1->Checked = true;
  List2->Checked = true;
}

//---------------------------------------------------------------------------
// Returns list of currently selected files in the file list.
// The list is compatible with SHFileOperation() function, i.e.
// it is represented by a long string. Each file name is terminated with
// a zero character. The list is terminated with two zero characters.
// The list is allocated in the heap. The caller should free it.
char *__fastcall TForm1::get_current_files(void)
{
  if ( ListView1->SelCount == 0 ) return NULL;
  AnsiString all = "";
  for ( int i=0; i < ListView1->Items->Count; i++ )
  {
    if (ListView1->Items->Item[i]->Selected )
    {
      if ( all.c_str()[0] != '\0' ) all = all + ";";
      all = all + (AnsiString(encrypted_folder) +
                                        ListView1->Items->Item[i]->Caption);
    }
  }
  int len = all.Length();
  char *file = new char[len+2];
  strcpy(file, all.c_str());
  file[len+1] = '\0';    // double nul for SHFileOperation()

  // replace all ';' by '\0'
  char *ptr = file;
  while ( 1 )
  {
    ptr = strchr(ptr, ';');
    if ( ptr == NULL ) break;
    *ptr++ = '\0';
  }
  return file;
}

//---------------------------------------------------------------------------
// Delete a file from the file list.
void __fastcall TForm1::DeleteExecute(TObject *Sender)
{
  char *file = get_current_files();
  if ( file != NULL )
  {
    SHFILEOPSTRUCT s;
    memset(&s, 0, sizeof(s));
    s.wFunc = FO_DELETE;
    s.pFrom = file;
    s.fFlags = FOF_ALLOWUNDO;
    SHFileOperation(&s);
    populate_list(encrypted_folder);          // refresh the list
    delete file;
  }
}

//---------------------------------------------------------------------------
// A file name was edited - rename the corresponding file on the disk.
void __fastcall TForm1::ListView1Edited(TObject *Sender, TListItem *Item,
      AnsiString &S)
{
  ((TAction*)(ActionList1->Actions[0]))->ShortCut = ShortCut(VK_DELETE, TShiftState());
  char *file = get_current_files();
  if ( file != NULL )
  {
    char to[MAXSTR];
    strcpy(stpcpy(to, encrypted_folder), S.c_str());
    to[strlen(to)+1] = '\0';    // double nul for SHFileOperation()
    SHFILEOPSTRUCT s;
    memset(&s, 0, sizeof(s));
    s.wFunc = FO_RENAME;
    s.pFrom = file;
    s.pTo   = to;
    s.fFlags = FOF_ALLOWUNDO;
    populate_list(encrypted_folder);
    SHFileOperation(&s);
    populate_list(encrypted_folder);
    delete file;
  }
}

//---------------------------------------------------------------------------
// When the user asks to rename a file, activate in-place editor.
void __fastcall TForm1::RenameExecute(TObject *Sender)
{
  ListView1->Selected->EditCaption();
}

//---------------------------------------------------------------------------
// Enable/disable some actions depending whether a file in the file list is selected
void __fastcall TForm1::ListView1SelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
  Rename->Enabled = Selected;
  Delete->Enabled = Selected;
  SendTo->Enabled = Selected;
}

//---------------------------------------------------------------------------
// This function prevents everything dragged from the file list to be
// dropped on the very file list. 
void __fastcall TForm1::ListView1DragOver(TObject *Sender, TObject *Source,
      int X, int Y, TDragState State, bool &Accept)
{
  Accept = false;
}

//-------------------------------------------------------
// process WM_SENTFILES message
// This message is sent by another copy of aCrypt, see acrypt.cpp
void __fastcall TForm1::WMSentFiles(TMessage &Msg)
{
  TMessage M;
  M.LParam = WM_LBUTTONDBLCLK;
  WMTrayNotify(M);
  DWORD read;
  DWORD len;
  if ( ReadFile(mail, &len, sizeof(len), &read, NULL) )
  {
    char *buf = new char[len];
    if ( !buf ) nomem("sentfiles");
    if ( ReadFile(mail, buf, len, &read, NULL) )  // encrypt the sent files
      encrypt(buf);
  }
}

//---------------------------------------------------------------------------
// Process the WM_DROPFILES message
void __fastcall TForm1::AppOnMessage(TMsg &Msg, bool &handled)
{
  switch ( Msg.message )
  {
    case WM_DROPFILES:
      {
        Application->BringToFront();
        HANDLE h = HANDLE(Msg.wParam);
        char file[MAXSTR];
        int cnt = DragQueryFile(h, 0xFFFFFFFF, NULL, 0);
        AnsiString files = "";
        for ( int i=0; i < cnt; i++ )
        {
          DragQueryFile(h, i, file, sizeof(file));
          Application->ProcessMessages();
          if ( strnicmp(file, encrypted_folder, strlen(encrypted_folder)) == 0 )
            continue;
          if ( files.c_str()[0] != 0 ) files += ";";
          files += file;
          handled = true;
        }
        encrypt(files.c_str());
        if ( handled ) DragFinish(h);
      }
      return;
  }
  handled = false;
}

//---------------------------------------------------------------------------
// Draw the file list with "SAFE STORE" on the background.
void __fastcall TForm1::ListView1CustomDraw(TCustomListView *Sender,
      const TRect &ARect, bool &DefaultDraw)
{
  if ( ListView1->ViewStyle == vsIcon || ListView1->ViewStyle == vsSmallIcon )
  {
    static char bktext1[] = "SAFE";
    static char bktext2[] = "STORE";
    ListView1->Canvas->FillRect(ARect);
    ListView1->Canvas->Font->Name = "Arial";
//    warning("color = %x", ListView1->Canvas->Brush->Color);
    ListView1->Canvas->Font->Color = TColor(RGB(0xF0, 0xF0, 0xF0));
    int cx = ListView1->Width / 2;
    int cy = ListView1->Height / 2;
    ListView1->Canvas->Font->Height = ListView1->Height / 2;
    TSize s;
    while ( 1 )
    {
      s = ListView1->Canvas->TextExtent(bktext2);
      if ( ListView1->Width > s.cx ) break;
      ListView1->Canvas->Font->Height -= 2;
    }
    s = ListView1->Canvas->TextExtent(bktext1);
    ListView1->Canvas->TextOut(cx-s.cx/2, cy-s.cy, bktext1);
    s = ListView1->Canvas->TextExtent(bktext2);
    ListView1->Canvas->TextOut(cx-s.cx/2, cy, bktext2);
  }
}

//---------------------------------------------------------------------------
// A callback function to display the progress of encryption
// Returns: true-ok, false-the user pressed "Cancel" button
static bool callback(int type, int cur, int all)
{
  const int done = 0;
  const int todo = 100;
  static int prevtype = -1;
  switch ( type )
  {
    case CB_ZIP:
      if ( prevtype != type ) Form3->Label1->Caption = a_compressing;
      break;
    case CB_FISH:
      if ( prevtype != type ) Form3->Label1->Caption = a_encrypting;
      break;
    case CB_STUB:
      if ( prevtype != type ) Form3->Label1->Caption = a_writing;
      break;
    case CB_END:
      break;
    default:
      error("interr cb %d", type);
  }
  prevtype = type;
  Form3->ProgressBar1->Position = done + cur * (todo-done) / all;
  Application->ProcessMessages();
  return Form3->Button1->Enabled;
}

//---------------------------------------------------------------------------
// Check if a file is ok to encrypt. Returns:
#define FG_OK        0   // ok
#define FG_EMPTY     1   // file is empty
#define FG_ENCRYPTED 2   // file is already encrypted
#define FG_DIR       3   // file is a directory
#define FG_NOFILE    4   // no such file

static int good_file(const char *fname)
{
  struct stat st;
  if ( stat(fname, &st) ) return FG_NOFILE;
  if ( st.st_mode & S_IFDIR ) return FG_DIR;
  if ( st.st_size == 0 ) return FG_EMPTY;
  FILE *fp = fopen(fname, "r");
  if ( fp != NULL )
  {
    // An encrypted file has a signature at its end. Check its presence.
    char buf[ENCRYPTED_SIGN_SIZE];
    fseek(fp, -ENCRYPTED_SIGN_SIZE, SEEK_END);
    fread(buf, 1, ENCRYPTED_SIGN_SIZE, fp);
    fclose(fp);
    if ( memcmp(buf, ENCRYPTED_SIGN, ENCRYPTED_SIGN_SIZE) == 0 )
      return FG_ENCRYPTED;
  }
  return FG_OK;
}

//---------------------------------------------------------------------------
// Build a "system" file name.
// A "system" file resides in the aCrypt directory
// For example, if aCrypt is in c:\program files\acrypt
// and file="somefile.txt", this function will return
// c:\program files\somefile.txt

char *sysfile(const char *file, char *buf)
{
  strcpy(buf,Application->ExeName.c_str());
  return strcpy((char*)basename(buf), file);
}

//---------------------------------------------------------------------------
// hide the "please wait" window and re-enable the main
static void hide_progress_window(void)
{
  if ( Form3->Visible )
  {
    EnableTaskWindows(WindowList);
    Form3->Visible = false;
  }
}

//---------------------------------------------------------------------------
// Check the error code and display an error message if necessary
// Returns: false-no error, true-the processing should stop because of error
bool show_error(int code, const char *arg)
{
  if ( code == ERR_OK ) return false;     // if ok, let' continue
  // there is an error, display it
  hide_progress_window();
  switch ( code )
  {
    case ERR_CANCEL:                      // cancel was pressed
      break;
    case ERR_NOFILE:                      // can't open file
      warning(a_cantopen, arg, GetLastError());
      break;
    case ERR_CRFILE:                      // can't create file
      warning(a_nocreate, arg, GetLastError());
      break;
    case ERR_READ:                        // can't read file
      warning(a_read, arg, GetLastError());
      break;
    case ERR_WRITE:                       // can't write file
      warning(a_write, arg, GetLastError());
      break;
    case ERR_NOMEM:                       // not enough memory
      nomem("encrypt");
      break;
    case ERR_STUB:                        // stub is not found
      warning(a_nostub);
      break;
    case ERR_BADSTUB:                     // bad stub - crc32 mismatch
      error(a_badstub, scrc32);
    default:                              // unknown error
      warning("Internal error %d", code);
      break;
  }
  return true;
}

//---------------------------------------------------------------------------
// Compress a file
// If it is a directory, compress its contents - all the files in it
// returns: true-ok, false-error occured, should stop
static bool zip_file_or_dir(const char *fname, const char *prefix)
{
  // Process pending windows messages so the interface is not stuck
  Application->ProcessMessages();
  char newpref[MAX_PATH];
  {
    char *pp = stpcpy(newpref, prefix);
    if ( pp != newpref ) *pp++ = '\\';
    strcpy(pp, basename(fname));            // create new prefix
  }
  struct stat st;
  stat(fname, &st);
  if ( st.st_mode & S_IFDIR )
  {
    WIN32_FIND_DATA fd;  // Structure for file data
    HANDLE h;            // Handle for searching
    char full[MAX_PATH];
    char *pf = stpcpy(full, fname);
    if ( pf != full ) *pf++ = '\\';

    strcpy(pf, "*.*");
    h = FindFirstFile(full, &fd);
    if (h != INVALID_HANDLE_VALUE) do
    {
      if ( fd.cFileName[0] == '.' )
      {
        if ( fd.cFileName[1] == '\0' ) continue;
        if ( fd.cFileName[1] == '.' && fd.cFileName[2] == '\0' ) continue;
      }
      strcpy(pf, fd.cFileName);            // create new filename
      if ( !zip_file_or_dir(full, newpref) )
      {
        FindClose(h);
        return false;
      }
    } while (FindNextFile(h, &fd));
    FindClose(h);
  }
  else // regular file
  {
    Form3->Caption = newpref;
    int code = zip_one_file(fname, newpref, callback);
    if ( show_error(code, fname) ) return false;
  }
  return true;
}

//---------------------------------------------------------------------------
// Encrypt files. The list of files is represented as a long string.
// A semicolon separates the file names.
// Returns: true-ok, false-error ocurred and displayed
static bool _encrypt(char *files)
{
  bool asked = false;
  bool multi = strchr(files, ';') != NULL;
  char target[MAXSTR];
  int nfiles = 0;
  for ( char *fname=strtok(files,";"); fname != NULL; fname=strtok(NULL, ";") )
  {
    if ( fname[0] == '\0' ) continue;
    // check if the file is ok
    int code = good_file(fname);
    switch ( code )
    {
      case FG_ENCRYPTED:            // already encrypted
        warning(a_encrypted, fname);
        return false;
      case FG_NOFILE:               // no such file
        warning(a_cantopen, fname);
        return false;
    }
    // if the questions are not asked yet, do it now
    if ( !asked )
    {
      asked = true;
      // check if the target already exists
      createdir(encrypted_folder);
      sprintf(target, "%s%s", encrypted_folder, basename(fname));
      char *ext = strrchr(target+strlen(encrypted_folder), '.');
      if ( ext == NULL ) ext = strchr(target, 0);
      strcpy(ext, ".exe");
      if ( access(target, 0) == 0
        && askyn(0, a_overwrite, target) <= 0 ) return false;
      // ok, ask for the passphrase
      askphrase->Caption = multi ? AnsiString(a_many)
                                 : AnsiString("Encrypt ") + fname;
      if ( !askphrase->CheckBox1->Checked )
      {
        askphrase->Edit1->Text = "";
        askphrase->Edit2->Text = "";
      }
      if ( askphrase->ShowModal() != mrOk ) return false;
      // save the passphrase if asked
      write_config(askphrase->CheckBox1->Checked, ctray, askphrase->Edit1->Text.c_str());
      // initialize the compression and pass it the attached note
      start_zip(askphrase->Memo1->Text.c_str());
      // disable the main form while encrypting
      WindowList = DisableTaskWindows(NULL);
      // display "please wait" window
      Form3->Button1->Enabled = true;
      Form3->Caption = basename(fname);
      Form3->Visible = true;
    }
    // compress each file in the list
    if ( !zip_file_or_dir(fname, "") ) return false;
    nfiles++;
  }
  if ( nfiles != 0 )
  {
    AnsiString &phrase = askphrase->Edit1->Text;
    // get the file with the stub - in fact, the external stub file is not
    // used in the final version of aCrypt. But we still leave it here just
    // in case.
    char stubbuf[MAXSTR];
    sysfile(stub, stubbuf);
    // terminate the compression and encrypt the result
    int code = finish_zip(target, phrase.c_str(), stubbuf, callback);
    hide_progress_window();
    if ( show_error(code, target) ) return false;
    // refresh the file list
    Form1->populate_list(encrypted_folder);
    // while this is not the best way to wipe the passphrase,
    // this is better than nothing:
    if ( !askphrase->CheckBox1->Checked )
    {
      askphrase->Edit1->Text = "";
      askphrase->Edit2->Text = "";
    }
  }
  return true; // success
}

//---------------------------------------------------------------------------
// Encrypt a file. Re-invocation safe version - if called recursively,
// silently returns. Nobody is going to call it recursively anyway.
// Returns: true-ok, false-an error occured and displayed
bool encrypt(char *files)
{
  static bool guard = false;
  if ( guard ) return false;
  guard = true;
  bool ok = _encrypt(files);
  guard = false;
  return ok;
}

//---------------------------------------------------------------------------
// If the user drags a file from the file list to another application,
// process it here. We use DragSource Delphi component to accompilsh
// the task.
void __fastcall TForm1::ListView1MouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  static bool AlreadyDragging;
  if ( AlreadyDragging ) return;

  //Make sure a mouse button is pressed before starting the drag ...
  //and the mouse has moved at least 10 pixels
  //(DragPoint is set in the ListView1MouseDown event)
  if ( !Shift.Contains(ssLeft) && !Shift.Contains(ssRight) ) return;


  //      ((abs(DragPoint.X - X) <10) and (abs(DragPoint.Y - Y) <10)) then
  //    exit;

  //If no files selected then exit...
  if ( ListView1->SelCount == 0 ) return;

  //Stop this event being re-executed once a drag has been initiated...
  AlreadyDragging = true;
  
  //Delete any filenames left from a previous dragdrop...
  DropFileSource1->Files->Clear();

  //Fill DropSource1.Files with selected filenames in ListView1...
  for ( int i=0; i < ListView1->Items->Count; i++ )
  {
    if (ListView1->Items->Item[i]->Selected )
      DropFileSource1->Files->Add(AnsiString(encrypted_folder) +
                                      ListView1->Items->Item[i]->Caption);
  }

  //Start the dragdrop...
  //Note: DropFileSource1.DragTypes = [dtCopy] set at design time.
  DropFileSource1->Execute();

  populate_list(encrypted_folder); // refresh the file list
  
  //Note: Execute does not return until the drop has finished -
  //either sucessfully with a copy,move of link operation or cancelled.
  AlreadyDragging = false;
}

//---------------------------------------------------------------------------
// A double click on a file icon open the file
void __fastcall TForm1::ListView1DblClick(TObject *Sender)
{
  if ( ListView1->SelCount == 0 ) return;
  ShellExecute(Handle, "open",
            (AnsiString(encrypted_folder)+ListView1->Selected->Caption).c_str(),
              "", NULL, SW_SHOWDEFAULT);
}
//---------------------------------------------------------------------------
// "Exit" from the context menu allows the user to exit the program
void __fastcall TForm1::ExitExecute(TObject *Sender)
{
  cfg_position(Left, Top, Width, Height);   // save the window position
  Application->Terminate();
}

//---------------------------------------------------------------------------
// The user clicked on the aCrypt image
void __fastcall TForm1::wwwExecute(TObject *Sender)
{
  provide_help();
}
//---------------------------------------------------------------------------
// Redraw the file list completely - this is required to redraw the background
// "SAFE STORE" text
void __fastcall TForm1::ListView1Resize(TObject *Sender)
{
  ListView1->Invalidate();
}
//---------------------------------------------------------------------------
// If the user wants to edit the file name, temporarily disable the Del key
// as the hotkey for Delete File.
void __fastcall TForm1::ListView1Editing(TObject *Sender, TListItem *Item,
      bool &AllowEdit)
{
  ((TAction*)(ActionList1->Actions[0]))->ShortCut = 0;
}
//---------------------------------------------------------------------------
// All forms of help requests lead to WWW based help.
bool __fastcall TForm1::FormHelp(WORD Command, int Data, bool &CallHelp)
{
  provide_help();
  CallHelp = false;
  return false;
}
//---------------------------------------------------------------------------
static void _USERENTRY showurl(void *)
{
  ShellExecute(Application->Handle, "open", url, NULL, NULL, SW_SHOWDEFAULT);
}

// Invoke a browser on a separate thread because it may take time to launch it.
// We don't want the interface to be frozen at this time.
void provide_help(void)
{
  _beginthread(showurl, 4096, NULL);
}
//---------------------------------------------------------------------------

 