//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <time.h>
#include "fish.h"
#include "unit1.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// Delete a registry key and all its subkeys
// This function behaves in the same manner on 95 and NT.
// It deletes the key and all its subkeys
// The source code taken from WDJ, May 1999
LONG WDJRegDeleteKey(HKEY hKeyRoot, LPCTSTR pszKey)
{
  HKEY hSubKey;
  LONG lRet;
  TCHAR szKeyName[MAX_PATH+1];
  DWORD cbName = sizeof(szKeyName);

  // Open the key root key
  if ( ERROR_SUCCESS != RegOpenKeyEx(hKeyRoot, pszKey, 0,
                                KEY_READ | KEY_WRITE, &hSubKey) )
    return ERROR_CANTOPEN;

  // Enumerate each key and call itself recursively
  do
  {
    lRet = RegEnumKey(hSubKey, 0, szKeyName, cbName);
    if ( lRet == ERROR_SUCCESS )
      lRet = WDJRegDeleteKey(hSubKey, szKeyName);
  } while ( lRet == ERROR_SUCCESS );

  // Close the registry key
  RegCloseKey(hSubKey);

  // Call the native RegDeleteKey() to actually delete the key
  return RegDeleteKey(hKeyRoot, pszKey);
}

//---------------------------------------------------------------------------
bool config_ok = false;   // found configuration information in the registry?
static const char key_run[]   = "Software\\Microsoft\\Windows\\CurrentVersion\\Run";
static const char key_sendto[]= "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders";
static const char root[]      = "Software\\Datarescue\\aCrypt";
static const char k_phrase[]  = "Phrase";
static const char k_seed[]    = "Seed";
static const char k_rand[]    = "Random";
static const char k_folder[]  = "Folder";
static const char k_ctray[]   = "CloseTray";                   // tray on close
static const char k_pos[]     = "Position";

// window position is saved in the following structure:
struct winpos_t
{
  int left;
  int top;
  int width;
  int height;
};

//---------------------------------------------------------------------------
// Get the main form position from the registry
void read_cfg_position(void)
{
  HKEY h;
  LONG code = RegOpenKeyEx(HKEY_CURRENT_USER, root, 0, KEY_READ, &h);
  if ( code == ERROR_SUCCESS )
  {
    DWORD len;
    winpos_t w;
    len = sizeof(w);
    code = RegQueryValueEx(h, k_pos, 0, NULL, (uchar*)&w, &len);
    if ( code == ERROR_SUCCESS )
    {
      Form1->Top   = w.top;
      Form1->Left  = w.left;
      Form1->Width = w.width;
      Form1->Height = w.height;
    }
    RegCloseKey(h);
  }
}

//---------------------------------------------------------------------------
// Read configuration information from the registry
void read_config(bool *remember, char *phrase)
{
  time_t now = time(NULL);
  HKEY h;
  *remember = false;
  LONG code = RegOpenKeyEx(HKEY_CURRENT_USER, root, 0, KEY_READ, &h);
  if ( code != ERROR_SUCCESS )
  {
SETRAND:
    // this is not very good - not enough randomness at the very first start
    SetRand(now);
    make_more_random();
    return;
  }
  {
    DWORD len;
    len = MAX_PATH;
    RegQueryValueEx(h, k_folder, 0, NULL, (uchar*)encrypted_folder, &len);
    len = sizeof(ctray);
    RegQueryValueEx(h, k_ctray, 0, NULL, LPBYTE(&ctray), &len);
    len = sizeof(randPtr);
    RegQueryValueEx(h, k_rand, 0, NULL, LPBYTE(&randPtr), &len);
    len = sizeof(randBits);
    code = RegQueryValueEx(h, k_seed, 0, NULL, LPBYTE(randBits), &len);
    if ( code != ERROR_SUCCESS )
    {
      RegCloseKey(h);
      goto SETRAND;
    }
    // check if the passphrase is saved in the registry.
    // if it is, it is encrypted
    len = MAXSTR;
    code = RegQueryValueEx(h, k_phrase, 0, NULL, LPBYTE(phrase), &len);
    if ( code == ERROR_SUCCESS )
    {
      // yes, it is here. decrypt it now using exactly the same IV as
      // was used in encryption.
      uchar iv[KEYSIZE];
      produce_iv(iv);
      // use value of "root" as the passphrase for encryption
      cipher(false, iv, (uchar *)phrase, len, root, NULL);
      *remember = true;
    }
    make_more_random();
  }
  config_ok = true;    // signal that the configuration info is successfully read
  RegCloseKey(h);
}

//---------------------------------------------------------------------------
// Write config into the registry
void write_config(bool remember, DWORD ctray, const char *phrase)
{
  HKEY h;
  DWORD disp;
  LONG code = RegCreateKeyEx(HKEY_CURRENT_USER, root, 0, 0,
                 REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &h, &disp);
  if ( code != ERROR_SUCCESS )
  {
    registry_error();
    return;
  }

  make_more_random();
  RegSetValueEx(h, k_ctray,0, REG_DWORD, LPBYTE(&ctray), sizeof(ctray));
  RegSetValueEx(h, k_seed, 0, REG_BINARY, LPBYTE(randBits), sizeof(randBits));
  RegSetValueEx(h, k_rand, 0, REG_DWORD, LPBYTE(&randPtr), sizeof(randPtr));
  // If asked to save the passphrase, encrypt it with
  // a predictable IV and a predictable passphrase.
  char encpass[MAXSTR];
  uchar iv[KEYSIZE];
  produce_iv(iv);
  int enclen = stpcpy(encpass, phrase) - encpass;
  if ( enclen < 64 ) enclen = 64;
  // use value of "root" as the passphrase for encryption
  cipher(true, iv, (uchar *)encpass, enclen, root, NULL);
  if ( remember )
    RegSetValueEx(h, k_phrase, 0, REG_BINARY, (uchar *)encpass, enclen); // save it
  else
    RegDeleteValue(h, k_phrase);  // delete the passphrase from the registry
  RegCloseKey(h);
}

//---------------------------------------------------------------------------
// Save the value of incoming directory into the registry
// Returns: true-ok, false-problem, error message is displayed
bool cfg_folder(const char *folder)
{
  HKEY h;
  DWORD disp;
  LONG code = RegCreateKeyEx(HKEY_CURRENT_USER, root, 0, 0,
                 REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &h, &disp);
  if ( code != ERROR_SUCCESS )
  {
    registry_error();
    return false;
  }
  RegSetValueEx(h, k_folder, 0, REG_SZ, (uchar*)folder, strlen(folder)+1);
  RegCloseKey(h);
  return true;
}

//---------------------------------------------------------------------------
// Save the current window position into the registry
// Returns: true-ok, false-problem, error message is displayed
bool cfg_position(int left, int top, int width, int height)
{
  HKEY h;
  DWORD disp;
  LONG code = RegCreateKeyEx(HKEY_CURRENT_USER, root, 0, 0,
                 REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &h, &disp);
  if ( code != ERROR_SUCCESS )
  {
    registry_error();
    return false;
  }
  winpos_t w;
  w.left = left;
  w.top  = top;
  w.width = width;
  w.height = height;
  RegSetValueEx(h, k_pos, 0, REG_BINARY, (uchar*)&w, sizeof(w));
  RegCloseKey(h);
  return true;
}

//---------------------------------------------------------------------------
// Enable/disable "autorun" feature - a possibility to run aCrypt when the system
// is started
void correct_autorun(bool use)
{
  HKEY h;
  LONG code = RegOpenKeyEx(HKEY_CURRENT_USER, key_run, 0, KEY_WRITE, &h);
  if ( code == ERROR_SUCCESS )
  {
    if ( use )
    {
      char buf[MAXSTR];
      sprintf(buf, "%s -m", Application->ExeName.c_str());  // -m=start minimized
      RegSetValueEx(h, appname, 0, REG_SZ, (uchar*)buf, strlen(buf)+1);
    }
    else
    {
      RegDeleteValue(h, appname);
    }
    RegCloseKey(h);
  }
}

//---------------------------------------------------------------------------
// Will the program autorun?
// Returns: true-yes
bool autorun_present(void)
{
  bool ok = false;
  HKEY h;
  LONG code = RegOpenKeyEx(HKEY_CURRENT_USER, key_run, 0, KEY_READ, &h);
  if ( code == ERROR_SUCCESS )
  {
    code = RegQueryValueEx(h, appname, 0, NULL, NULL, NULL);
    if ( code == ERROR_SUCCESS ) ok = true;
    RegCloseKey(h);
  }
  return ok;
}

//---------------------------------------------------------------------------
// Get the name of the link file which will be run by "SendTo" shell menu item.
static bool get_sendto_link(char *folder, size_t size)
{
  HKEY h;
  bool ok = false;
  // get the usual place for shell links
  LONG code = RegOpenKeyEx(HKEY_CURRENT_USER, key_sendto, 0, KEY_READ, &h);
  if ( code == ERROR_SUCCESS )
  {
    DWORD len = size;
    // get the usual place for "Send to..." links
    code = RegQueryValueEx(h, "SendTo", 0, NULL, (uchar *)folder, &len);
    if ( code == ERROR_SUCCESS )
    {
      // append the program name there
      char *ptr = strchr(folder, '\0');
      if ( ptr[-1] != '\\' ) *ptr++ = '\\';
      strcpy(stpcpy(ptr, appname), ".lnk");
      ok = true;
    }
    RegCloseKey(h);
  }
  return ok;
}

//---------------------------------------------------------------------------
// CreateLink - uses the shell's IShellLink and IPersistFile interfaces
//   to create and store a shortcut to the specified object.
// Returns the result of calling the member functions of the interfaces.
// lpszPathObj - address of a buffer containing the path of the object
// lpszPathLink - address of a buffer containing the path where the
//   shell link is to be stored
// lpszDesc - address of a buffer containing the description of the
//   shell link

static HRESULT CreateLink(LPCSTR lpszPathObj,
    LPSTR lpszPathLink, LPSTR lpszDesc)
{
    HRESULT hres;
    IShellLink *psl;

    // Get a pointer to the IShellLink interface.
    hres = CoCreateInstance(CLSID_ShellLink, NULL,
        CLSCTX_INPROC_SERVER, IID_IShellLink, (void**)&psl);
    if (SUCCEEDED(hres)) {
        IPersistFile* ppf;

        // Set the path to the shortcut target, and add the
        // description.
        psl->SetPath(lpszPathObj);

        psl->SetDescription(lpszDesc);

        psl->SetIconLocation(lpszPathObj, 0);

       // Query IShellLink for the IPersistFile interface for saving the
       // shortcut in persistent storage.
        hres = psl->QueryInterface(IID_IPersistFile, (void**)&ppf);

        if (SUCCEEDED(hres)) {
            wchar_t wsz[MAX_PATH];

            // Ensure that the string is ANSI.
            MultiByteToWideChar(CP_ACP, 0, lpszPathLink, -1,
                wsz, MAX_PATH);


            // Save the link by calling IPersistFile::Save.
            hres = ppf->Save(wsz, TRUE);
            ppf->Release();
        }
        psl->Release();
        CoUninitialize();
    }
    return hres;
}

//---------------------------------------------------------------------------
// Create/delete .lnk for "SendTo"
void correct_sendto(bool use)
{
  char folder[MAX_PATH];    // name of file with the sendto links
  if ( get_sendto_link(folder, sizeof(folder)) )
  {
    if ( use )
      CreateLink(Application->ExeName.c_str(), folder, appname);
    else
      unlink(folder);
  }
}

//---------------------------------------------------------------------------
// Is "SendTo" link file present?
bool sendto_present(void)
{
  char folder[MAX_PATH];    // name of file with the sendto links
  if ( get_sendto_link(folder, sizeof(folder)) )
    return access(folder, 0) == 0;
  return false;
}


