#include <vcl.h>
#pragma hdrstop
#include "fish.h"
#include <time.h>
#include "md5.h"

#define ROL(x,n) (((x) << ((n) & 0x1F)) | ((x) >> (32-((n) & 0x1F))))

unsigned long randBits[64]= {1};  /* use Knuth's additive generator */
int randPtr;
/*
+*****************************************************************************
*
* Function Name:    Rand
*
* Function:         Generate random number
*
* Arguments:        None.
*
* Return:           New random number.
*
* Notes:            Uses Knuth's additive generator, other magic
*
-****************************************************************************/
unsigned long Rand(void)
    {
    if (randPtr >= 57)
        randPtr = 0;            /* handle the ptr wrap */

    randBits[randPtr] += randBits[(randPtr < 7) ? randPtr-7+57 : randPtr-7];

    randBits[62]+= randBits[61];
    randBits[63] = ROL(randBits[63],9) + 0x6F4ED7D0;    /* very long period! */
    
    return (randBits[randPtr++] ^ randBits[63]) + randBits[62];
    }


//---------------------------------------------------------------------------
static void finish_initializing(int x)
{
  if ((x & 1) == 0)   /* insure maximal period by having at least one odd value */
      randBits[0]++;

  for (int i=0;i<1000;i++)
      Rand();         /* run it for a while */

  randBits[63] = Rand();
  randBits[62] = Rand();
  randBits[61] = Rand() | 1;  /* make it odd */
}

/*
+*****************************************************************************
*
* Function Name:    SetRand
*
* Function:         Initialize random number seed
*
* Arguments:        seed    =   new seed value
*
* Return:           None.
*
* Notes:
*
-****************************************************************************/
void SetRand(unsigned long seed)
    {
    int i;
    unsigned long x;

    randPtr=0;
    for (i=x=0;i<64;i++)
        {
        randBits[i]=seed;
        x |= seed;      /* keep track of lsb of all entries */
        seed = ROL(seed,11) + 0x12345678;
        }

    finish_initializing(x);
}

//---------------------------------------------------------------------------
static void calc_random_hash(uchar *hash)
{
  struct MD5Context mdContext;
  MD5Init(&mdContext);
  ulong pid = GetCurrentProcessId();
  ulong tid = GetCurrentThreadId();
  time_t t = time(0);
  ulong ticks = GetTickCount();
  POINT mouse;
  GetCursorPos(&mouse);
  MEMORYSTATUS ms;
  GlobalMemoryStatus(&ms);

  MD5Update(&mdContext, (uchar *)&pid, sizeof(ulong));
  MD5Update(&mdContext, (uchar *)&tid, sizeof(ulong));
  MD5Update(&mdContext, (uchar *)&t, sizeof(time_t));
  MD5Update(&mdContext, (uchar *)&ticks, sizeof(ulong));
  MD5Update(&mdContext, (uchar *)&mouse, sizeof(mouse));
  MD5Update(&mdContext, (uchar *)&ms, sizeof(ms));
  MD5Final(hash, &mdContext);
}

//---------------------------------------------------------------------------
void make_more_random(void)
{
  uchar hash[16];
  calc_random_hash(hash);
  ulong *ptr = (ulong *)hash;
  for ( int i=0; i < 64; i += 4 )
  {
    randBits[i+0] ^= ptr[0];
    randBits[i+1] ^= ptr[1];
    randBits[i+2] ^= ptr[2];
    randBits[i+3] ^= ptr[3];
  }
  finish_initializing(0);
}

