
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <io.h>

#include "md5.h"
#include "aes.h"


const int MAXSTR  = 1024;
const int KEYBITS = 128;
const int KEYSIZE = 16;         // in bytes

typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned long  ulong;

//
// the encrypted data looks like this:
//      header_t
//      message
//x     header2_t
//x     fileh_t     -+
//x     file name    |   many times
//x     ciphertext  -+
//      "ENCRYPTED_FILE"
//

struct header2_t;
struct fileh_t;

struct header_t
{
  ulong magic;
#define MAGIC	0x76543210
  ulong crc32;          // crc32 of the encrypted file excluding first 8 bytes
  ulong size;           // size of encrypted file excluding header
  ulong stub_crc32;     // CRC32 of the stub
  uchar iv[KEYSIZE];    // initialization vector
  ulong msglen;         // length of the message (multiple of 4)
  
  char *crc_start(void) { return (char *)this + 8; }
  ulong crc_len(void)   { return size + (sizeof(header_t) + msglen - 8); }
  char *msg(void) { return (char *)(this+1); }
  uchar *ct(void) { return (uchar *)(msg()+msglen); }
  header2_t *h2(void) { return (header2_t *)ct(); }
};

struct header2_t
{
  uchar chk[2];         // first 16 bits of iv
  ushort nfiles;        // number of files in the archive
  fileh_t *files(void) { return (fileh_t *)(this+1); }
};

struct fileh_t
{
  ulong csize;          // compressed size
  ulong usize;          // uncompressed size
  FILETIME atime, ctime, mtime;
  uchar namelen;
  char *name(void)    { return (char *)(this+1); }
  uchar *text(void)   { return (uchar *)(name()+namelen); }
  fileh_t *next(void) { return (fileh_t *)(text()+csize); }
};

                                 // 12345678901234
#define      ENCRYPTED_SIGN        "ENCRYPTED_FILE"
#define      ENCRYPTED_SIGN_SIZE   14

void warning(const char *format, ...);
void error(const char *format, ...);
int askyn(int def, const char *format, ...);
void nomem(const char *where);
const char *basename(const char *fname);

char *b2a(const uchar *b, char *a); // binary to ascii

void start_zip(char *message);
int zip_one_file(const char *file, const char *filename, bool callback(int,int,int));
int finish_zip(const char *outfile, const char *phrase,
                        const char *stub, bool callback(int,int,int));

// low level routines for config:
void produce_iv(uchar *iv);
int cipher(bool encrypt,uchar *iv, uchar *bytes, int len, const char *phrase,
                                        bool callback(int));
#define ERR_OK     0
#define ERR_NOFILE -1
#define ERR_CRFILE -2
#define ERR_READ   -3
#define ERR_WRITE  -4
#define ERR_NOMEM  -5
#define ERR_BADCRC -6
#define ERR_DECR   -7
#define ERR_STUB   -8
#define ERR_BADSTUB -9
#define ERR_CANCEL -10

#define CB_ZIP     0    // progressbar callback types
#define CB_FISH    1
#define CB_STUB    2
#define CB_END     3

// from random.cpp
extern DWORD randBits[64];
extern int randPtr;

void SetRand(DWORD seed);
DWORD Rand(void);
void make_more_random(void);

ulong crc32(void *buf, int size);

void hashpass(const char *keystring, int keylen, uchar *hash);

extern char appname[];

void createdir(char *dir);

bool BrowseForFolder(HWND hwnd, const char *title, char *buf);

